Confluence.CreateJiraContent.FormHelper = (function($, _) {
    var DEFAULT_OPTION_VALUE = "-1", DEFAULT_LOADING_VALUE = "loading",
        DEFAULT_OPTION_TEXT = AJS.I18n.getText("createjiracontent.dialog.form.select.default.text"),
        DEFAULT_OPTION_PROJECT_TEXT = AJS.I18n.getText("createjiracontent.dialog.form.select.default.project.text"),
        DEFAULT_OPTION_ISSUETYPE_TEXT = AJS.I18n.getText("createjiracontent.dialog.form.select.default.issuetype.text"),
        LOADING_OPTION_TEXT = AJS.I18n.getText("createjiracontent.dialog.form.select.loading.text"),
        BOTTOM_ARROW_CLASS = 'aui-bottom-arrow-color';
    var epicIssueTypeId = null;
    var $currentRequiredFields = null;
    var hasUnsupportedFields = false;

    var DEFAULT_REQUIRED_FIELDS = ["project", "issuetype", "summary", "reporter"];
    // Status of description field
    var currentDescriptionFieldStatus;

    // store the form controls to avoid finding it many times
    var createIssueFormControls;
    var $createIssueForm;
    var defaultIssueTypeNames;

    // XHR object, hold this object to call abort function when needed
    var xhrProject, xhrEpicPage;

    // Declare an instance of Confluence.storageManager
    var storageManager = Confluence.storageManager("jira-content");
    var storedDialogValues;
    var epicSummaryText;
    // find and store frequently used controls
    var init = function($form) {
        createIssueFormControls = {
            $serverSelect : $form.find("#jira-servers"),
            $projectSelect : $form.find("#jira-projects"),
            $issueTypeSelect : $form.find("#jira-issue-types"),
            $projectSpinner : $form.find("#projectSpinner"),
            $hiddenItems : $form.children().not(".dialog-header, #jiraserver-issue-messages"),
            $requiredFieldsPanel : $form.find("#jira-required-fields-panel"),
            $summaryField : $form.find("#issue-summary"),
            $descriptionField : $form.find("#issue-description"),
            $jiraEpicContent : $form.find("#jira-epic-content")
        };
        $createIssueForm = $form;
        // Get default issue type names from i18n for set default display of issue type select box
        defaultIssueTypeNames = AJS.I18n.getText("createjiracontent.dialog.form.issuetype.default.text").toLowerCase();
        defaultIssueTypeNames = defaultIssueTypeNames.split(",");
        //disable issueTypeSelect
        clearAndDisableSelectBox(createIssueFormControls.$issueTypeSelect);
        // show description field as first time 
        currentDescriptionFieldStatus = Confluence.CreateJiraContent.JiraIssue.FieldStatus.VISIBLE;
    };

    // Declare name of Storege Key as a constant
    var FORM_VALUE_STORAGE_KEY = "savedFormValues";

    /**
     * By default, press ESC button will reset form in IE, it causes some problem with select2 
     * (value change to default but text holder is still display the old value)
     * this function will prevent form reset in IE. But still keep features: close selectbox, close dialog by using ESC
     */
    var preventEscKeyResetFormInIE = function() {
        if($.browser.msie) {
            $createIssueForm.find("input, textarea").keydown(function(e) {
                if (e.keyCode == 27) { //ESC key
                    return false;
                }
            });
        }
    };

    /*** Select box ****/
    var clearAndAddDefaultOptionForSelectBox = function($selectBox, customizeOptionText) {
        var optionContent = aui.form.optionOrOptgroup({
            "value": DEFAULT_OPTION_VALUE,
            "text": customizeOptionText ? customizeOptionText : DEFAULT_OPTION_TEXT
        });
        $selectBox.html(optionContent);
        $selectBox.auiSelect2("val", DEFAULT_OPTION_VALUE);
    };

    var showLoadingAndDisableProjectSelectBox = function() {
        var $projectSelect = createIssueFormControls.$projectSelect;
        $projectSelect.auiSelect2("enable", false);
        var optionContent = aui.form.optionOrOptgroup({
            "value": DEFAULT_OPTION_VALUE,
            "text": LOADING_OPTION_TEXT
        });
        $projectSelect.html(optionContent);
        $projectSelect.auiSelect2("val", LOADING_OPTION_TEXT);
    };

    var enableProjectSelectBox = function() {
        createIssueFormControls.$projectSelect.auiSelect2("enable", true);
    };

    var displayErrorLoadingForSelectBox = function($selectBox) {
        clearAndAddDefaultOptionForSelectBox($selectBox);
    };

    var createSelect2WithTooltip = function($element, config) {
        var defaultConfig = {
            escapeMarkup: function(markup) {
                // Keep this for now.
                // Although automatic escaping of HTML in Confluence Select2 library is now disabled and this code currently does not do anything,
                // it will be needed if the select2 library is updated and re-enabled the autoescaping in the future.
                return markup;
            },
            formatResult: tooltipFormat,
            formatSelection: tooltipFormat
        };
        $element.auiSelect2($.extend(defaultConfig, config));
    };

    /*** Get & load data ***/
    var fillAndDisplayServers = function(servers) {
        var $serverSelect = createIssueFormControls.$serverSelect;
        var selectHTML = [];
        $.each(servers, function(index, server) {
            var optionContent = aui.form.optionOrOptgroup({
                "value": server.id,
                "text": server.name
            });
            selectHTML.push(optionContent);
        });
        $serverSelect.append(selectHTML.join(''));
        if (servers.length == 1) {
            $serverSelect.hide();
        } else {
            createSelect2WithTooltip(createIssueFormControls.$serverSelect,
                {
                    "containerCssClass": "select-container select-server-container",
                    "dropdownCssClass": "server-dropdown",
                    "width": "180px",
                    "minimumResultsForSearch": -1
                }
           );
        }
        // If storedDialogValues is defined, bind it into Select
        if (storedDialogValues) {
            setSummaryVisible(setSelectFieldValue($serverSelect, storedDialogValues.serverId));
        } else {
            setSummaryVisible(false);
        }
    };

    var loadServers = function() {
        return Confluence.CreateJiraContent.DataHelper.loadServers().done(function(servers) {
            if (servers.length) {
                fillAndDisplayServers(servers);
            } else {
                showJiraServerWarning(AJS.I18n.getText("createjiracontent.dialog.form.get.jiraserver.error.empty"));
            }
        });
    };

    var fillProjectSelectBox = function(projects) {
        var $projectSelect = createIssueFormControls.$projectSelect;
        clearAndAddDefaultOptionForSelectBox($projectSelect, DEFAULT_OPTION_PROJECT_TEXT);
        var selectHTML = [];
        $.each(projects, function(index, project) {
            var optionContent = aui.form.optionOrOptgroup({
                "value": project.id,
                "text": project.name
            });
            selectHTML.push(optionContent);
        });
        $projectSelect.append(selectHTML.join(''));
    };
    
    var updateDialogAfterLoadProjects = function(serverId, projects, bindStoreValueFunction) {
        detectEpicIssueOnPage(serverId);
        fillProjectSelectBox(projects);
        enableProjectSelectBox();
        bindStoreValueFunction();
        AJS.trigger('confluence-jira-content.form-updated');
    };

    var loadProjects = function() {
        var currentServer = getCurrentServer();
        var serverSupported = verifySupportedServer(currentServer);
        if (serverSupported) {
            var bindStoreValue = function() {
                // Sometime two separate servers have the same projects id, we have to make sure they match both server and issueType before show Summary
                if (storedDialogValues && storedDialogValues.serverId == currentServer.id) {
                    // If storedDialogValues is defined, bind it into Select
                    var success = setSelectFieldValue(createIssueFormControls.$projectSelect, storedDialogValues.projectId);
                    // In-case storedDialogValues is defined, but its value is not found in project list, so that value is not valid
                    setSummaryVisible(success);
                    // If project binding successful, IssueType should be loaded
                    if (success) {
                        loadIssueTypes();
                        checkCreateIssueFields();
                    }
                } else {
                    // Hide and reset IssueType Field
                    setSummaryVisible(false);
                    clearAndDisableSelectBox(createIssueFormControls.$issueTypeSelect);
                }
            };

            if (currentServer && currentServer.authUrl) {
                jiraServerStatus(401); // unauthorized
            } else {
                showLoadingAndDisableProjectSelectBox();
                return Confluence.CreateJiraContent.DataHelper.loadProjects(currentServer.id)
                    .done(function(projects) {
                        if (projects.errors && projects.errors.length) {
                            var err = projects.errors[0];
                            if (err.authenticationUri) {
                                jiraServerStatus(401);
                            } else {
                                showJiraServerWarning(err.message);
                            }
                            displayErrorLoadingForSelectBox(createIssueFormControls.$projectSelect);
                            enableProjectSelectBox();
                        } else {
                            jiraServerStatus(200, projects);
                            updateDialogAfterLoadProjects(currentServer.id, projects, bindStoreValue);
                        }
                    })
                    .fail(function(xhr) {
                        jiraServerStatus(xhr.status);
                        displayErrorLoadingForSelectBox(createIssueFormControls.$projectSelect);
                        enableProjectSelectBox();
                    })
            }
        }
    };

    // Check issue type name equals with default issue type name  
    var isMatchIssueTypeDefaultName = function(issueTypeName) {
        for ( var i = 0; i < defaultIssueTypeNames.length; i++) {
            if (defaultIssueTypeNames[i] == issueTypeName.toLowerCase()) {
                return true;
            }
        }
        return false;
    };

    var clearAndDisableSelectBox = function($selectBox) {
        $selectBox.auiSelect2("enable", false);
        clearAndAddDefaultOptionForSelectBox($selectBox, DEFAULT_OPTION_ISSUETYPE_TEXT);
        // make selectBox not be focused
        $selectBox.prev().find("a").removeAttr("tabindex");
    };

    var enableSelectBox = function($selectBox) {
        $selectBox.auiSelect2("enable", true);
        $selectBox.prev().find("a").attr("tabindex", "0");
    };

    var loadIssueTypes = function() {
        var projectId = createIssueFormControls.$projectSelect.val();
        var $issueTypeSelect = createIssueFormControls.$issueTypeSelect;
        if (projectId && projectId != DEFAULT_OPTION_VALUE) {
            enableSelectBox($issueTypeSelect);
            fillIssueTypeSelectBox($issueTypeSelect);

            // If storedDialogValues is defined, bind it into Select
            if (storedDialogValues) {
                setSummaryVisible(setSelectFieldValue($issueTypeSelect, storedDialogValues.issueTypeId));
            } else {
                setSummaryVisible(false);
            }
        } else {
            clearAndDisableSelectBox($issueTypeSelect);
        }
    };

    var getIssueTypes = function() {
        return Confluence.CreateJiraContent.DataHelper.getIssueTypes(createIssueFormControls.$serverSelect.val(),
            createIssueFormControls.$projectSelect.val());
    };

    var fillIssueTypeSelectBox = function($issueTypeSelect) {
        var selectedValue = null;
        var selectHtml = [];

        var issueTypes = getIssueTypes();
        $.each(issueTypes, function(index, issueType) {
            // store issueType id for make default value of issuetype selectbox
            if (!selectedValue && isMatchIssueTypeDefaultName(issueType.name)) {
                selectedValue = issueType.id;
            }

            var optionContent = aui.form.optionOrOptgroup({
                value: issueType.id,
                text: issueType.name
            });

            selectHtml.push(optionContent);
        });
        $issueTypeSelect.html(selectHtml.join(''));
        $issueTypeSelect.auiSelect2("val", selectedValue);

        // show default select when data is empty
        if (_.isEmpty(issueTypes)) {
            clearAndAddDefaultOptionForSelectBox($issueTypeSelect, DEFAULT_OPTION_ISSUETYPE_TEXT);
        }
    };

    //****** Create issue field functions *******/
    
    var showFormWaiting = function(waitingMessage) {
        enableSubmitButton(false);
        $createIssueForm.find("#form-message-spinner").addClass("aui-icon aui-icon-wait");
        $createIssueForm.find("#form-message").text(waitingMessage);
    };

    var removeFormWaiting = function() {
        $createIssueForm.find("#form-message-spinner").removeClass("aui-icon aui-icon-wait");
        $createIssueForm.find("#form-message").empty();
    };

    var removeUnsupportedFieldsMessage = function() {
        createIssueFormControls.$summaryField.parent().show();
        $createIssueForm.find("#create-issue-form-messages").empty().addClass("hidden");
        $createIssueForm.find("#issue-content, #jira-epic-content").removeClass("hidden");
        refreshDialog();
    };

    var handleDescriptionField = function() {
        if (currentDescriptionFieldStatus === Confluence.CreateJiraContent.JiraIssue.FieldStatus.HIDDEN) {
            createIssueFormControls.$descriptionField.parent().hide();
        } else if (!hasUnsupportedFields) {
            if (currentDescriptionFieldStatus === Confluence.CreateJiraContent.JiraIssue.FieldStatus.REQUIRED) {
                createIssueFormControls.$descriptionField.siblings('label').children('span').show();
            } else {
                createIssueFormControls.$descriptionField.siblings('label').children('span').hide();
            }
            createIssueFormControls.$descriptionField.parent().show();
        }
        refreshDialog();
    };

    var showUnsupportedFieldsMessage = function(unsupportedFields) {
        hasUnsupportedFields = true;
        var $formMessages = $createIssueForm.find("#create-issue-form-messages");
        var message = Confluence.CreateJiraContent.renderUnsupportedFieldsWarningMessage({
            unsupportedFields: _.map(unsupportedFields, function(item) {
                return item.name;
            }),
            projectLinkUrl: getCurrentJiraCreateIssueUrl()
        });
        AJS.messages.warning($formMessages, {
            body: message,
            closeable: false
        });
        Confluence.CreateJiraContent.Analytics.sendAnalyticsForRequiredFields();
        Confluence.CreateJiraContent.Analytics.bindAnalyticsForCreateDirectlyInJira();
        createIssueFormControls.$summaryField.parent().hide();
        createIssueFormControls.$descriptionField.parent().hide();

        $formMessages.removeClass("hidden");
        refreshDialog();
    };

    var renderRequiredFields = function(otherRequiredFields) {
        jiraIntegration.fields.renderCreateRequiredFields(
            createIssueFormControls.$requiredFieldsPanel,
            null,
            {
                serverId: createIssueFormControls.$serverSelect.val(),
                projectKey: getCurrentProject().key,
                issueType: createIssueFormControls.$issueTypeSelect.val()
            },
            {
                requiredFields: otherRequiredFields
            },
            _.bind(showUnsupportedFieldsMessage, this) // provide current scope for this function
        );
        // update current required fields
        $currentRequiredFields = createIssueFormControls.$requiredFieldsPanel.find(".jira-field").children('input,select,textarea');
        bindRequiredFieldEvents();
    };

    var checkCreateIssueFields = function() {
        removeUnsupportedFieldsMessage();
        $currentRequiredFields = null;
        hasUnsupportedFields = false;
        createIssueFormControls.$requiredFieldsPanel.empty();

        if(createIssueFormControls.$projectSelect.val() !== DEFAULT_OPTION_VALUE) {
            var onGetCreateIssueMetaSuccess = function(createMeta) {
                currentDescriptionFieldStatus = createMeta.descriptionFieldStatus;
                if (!_.isEmpty(createMeta.requiredFields)) {
                    renderRequiredFields(createMeta.requiredFields);
                }
                handleEpicContent(createMeta.epicField);
                removeFormWaiting();
                AJS.trigger("confluence-jira-content.form-updated");
            };

            var onCreateIssueMetaError = function(errorMessage) {
                removeFormWaiting();
            };

            showFormWaiting(AJS.I18n.getText("createjiracontent.dialog.create.issue.discover.fields.message.checking"));

            Confluence.CreateJiraContent.JiraIssue.getCreateIssueMeta(createGetFieldsParamObj(), onGetCreateIssueMetaSuccess, onCreateIssueMetaError);
            refreshDialog();
        } else {
            handleEpicContent();
        }
    };

    /**
     * Epic link can be configured to not display on screen, in this case, remove link suggestion.
     */
    var handleEpicContent = function(epicField) {
        $.when(xhrProject, xhrEpicPage).done(function() {
            var showEpic = epicIssueTypeId && !hasUnsupportedFields // do not display epic content when there are unsupported fields
            && ((createIssueFormControls.$projectSelect.val() === DEFAULT_OPTION_VALUE) // project has not chosen yet
                || (epicField && epicIssueTypeId !== createIssueFormControls.$issueTypeSelect.val())); // had epic field and does not choose the Epic issue type.

            createIssueFormControls.$jiraEpicContent.toggle(showEpic ? true : false);
        });
    };

    //****** Create issue field functions *******/

    // Check whether selected JIRA Server is supported version
    var verifySupportedServer = function(server) {
        if (!server.supportedVersion) {
            createIssueFormControls.$hiddenItems.addClass("hidden");
            showJiraServerWarning(AJS.format(AJS.I18n.getText('createjiracontent.messages.server.notsupported'), server.url));
        } else {
            createIssueFormControls.$hiddenItems.removeClass("hidden");
        }

        if ($createIssueForm.find('#text-suggestion').length) {
            $('#inline-dialog-create-issue-dialog > #arrow-create-issue-dialog').addClass(BOTTOM_ARROW_CLASS);
        }

        refreshDialog();
        return server.supportedVersion;
    };

    var bindElementEvents = function() {
        createIssueFormControls.$serverSelect.change(function(e) {
            $("#jiraserver-issue-messages").empty(); //clear the error message if have
            var currentServer = Confluence.CreateJiraContent.DataHelper.getServerById(e.val);
            var serverSupported = verifySupportedServer(currentServer);
            if (serverSupported) {
                createIssueFormControls.$jiraEpicContent.empty();
                createIssueFormControls.$hiddenItems.removeClass("hidden");
                setSummaryVisible(false);
                // In case user select other project which not match with our value, storedDialogValues is set to False, Then they re-select a valid one we have to reset storedDialogValues
                storedDialogValues = loadFormValues();
                $.when(loadProjects()).done(function() {
                    // We only clear and disable Issue Type if storedDialogValues is not defined. If not it will clear all storevalue binding
                    if (!storedDialogValues) {
                        clearAndDisableSelectBox(createIssueFormControls.$issueTypeSelect);
                        changeIssuesTypeIconForPreviewPanel($createIssueForm.find("#jira-issue-types option:selected").val());
                    }
                    createIssueFormControls.$projectSelect.change(); //trigger the event for projectSelect change.
                });
            }
        });

        createIssueFormControls.$projectSelect.change(function(e) {
            // Check if selected Project is match with
            var currentProjectValue = createIssueFormControls.$projectSelect.val();
            if (storedDialogValues && currentProjectValue != storedDialogValues.projectId) {
                storedDialogValues = false;
            }
            loadIssueTypes();
            Confluence.CreateJiraContent.CreateIssue.fillPreviewIssuesFromTable();
            changeIssuesTypeIconForPreviewPanel($createIssueForm.find("#jira-issue-types option:selected").val());
            AJS.trigger('confluence-jira-content.form-updated');
            checkCreateIssueFields();
        });
        
        createIssueFormControls.$issueTypeSelect.change(function(e) {
            changeIssuesTypeIconForPreviewPanel(e.val);
            AJS.trigger('confluence-jira-content.form-updated');
            checkCreateIssueFields();
        });

        $createIssueForm.on('keyup', createIssueFormControls.$summaryField, function(e) {
            AJS.trigger('confluence-jira-content.form-updated');
        });
        
        $createIssueForm.on('keyup', createIssueFormControls.$descriptionField, function(e) {
            AJS.trigger('confluence-jira-content.form-updated');
        });

        if (!Confluence.CreateJiraContent.CreateIssue.getDialogObject()) {
            AJS.bind('confluence-jira-content.form-updated', function() {
                enableSubmitButton(validateCreateIssueForm());
                handleDescriptionField();
            });

            AJS.bind('confluence-jira-content.form-switched', function() {
                handleDescriptionField();
            });
        }

        // Bind Edit Link Event
        $("#jira-content-create-issue-form .dialog-content").on("click", ".summary-stored-value a", function() {
            setSummaryVisible(false);
            storedDialogValues = false;

            return false;
        });
        preventEscKeyResetFormInIE();
    };

    var bindRequiredFieldEvents = function() {
        $currentRequiredFields.on('change', function(e) {
            AJS.trigger('confluence-jira-content.form-updated');
        });

        // The block of code bellow to override the behavior of onSelect function.
        // As default, datepicker doesn't fire change event, this override will fix this issue.
        // This override only apply for AUI versions which < 5.4.5
        if (AJS.version <= '5.4.5' ) {
            var $datepickers = $createIssueForm.find('input[data-aui-dp-uuid]');
            $datepickers.each(function(index, element) {
                var $dp = $(element),
                    uuid = $dp.attr('data-aui-dp-uuid');
                $dp.on('click', function() {
                    var $dpPopup = $('[data-aui-dp-popup-uuid='+uuid+']');
                    var defaultOnSelectHandler = $dpPopup.datepicker('option', 'onSelect');
                    $dpPopup.datepicker('option', 'onSelect', function(dateText, inst) {
                        defaultOnSelectHandler(dateText, inst);
                        $dp.change();
                    });
                    $dpPopup.parents('.aui-inline-dialog').addClass('datepicker-patch');
                });
            });
        }
    };

    var enableSubmitButton = function(enable) {
        if (enable) {
            $('.create-issue-submit').removeAttr('disabled');
        } else {
            $('.create-issue-submit').attr('disabled', 'disabled');
        }
    };

    /*** Validate ***/

    var validateCreateIssueForm = function() {
        var isFieldValueValid = function($field, invalidValue) {
            return ($.trim($field.val()) !== invalidValue);
        };

        var getInvalidRequiredField = function() {
            var isPlaceholderSupported = 'placeholder' in document.createElement('input');
            return _.find($currentRequiredFields, function(requiredElement) {
                var $field = $(requiredElement),
                    value = $.trim($field.val());
                if (isPlaceholderSupported) {
                    return !value;
                } else {
                    var placeholder = $field.attr('placeholder');
                    return (!value || value == placeholder);
                }
            });
        };

        var valid = isFieldValueValid(createIssueFormControls.$projectSelect, DEFAULT_OPTION_VALUE) &&
                    isFieldValueValid(createIssueFormControls.$issueTypeSelect, DEFAULT_OPTION_VALUE) &&
                    !hasUnsupportedFields &&
                    !getInvalidRequiredField();

        if (Confluence.CreateJiraContent.CreateIssue.isInTextMode() && valid) {
            valid = isFieldValueValid(createIssueFormControls.$summaryField, "");

            if (currentDescriptionFieldStatus === Confluence.CreateJiraContent.JiraIssue.FieldStatus.REQUIRED) {
                valid = isFieldValueValid(createIssueFormControls.$descriptionField, "");
            }
        }

        return valid;
    };

    var changeIssuesTypeIconForPreviewPanel = function(issueTypeId) {
        if (issueTypeId != -1) {
            var issueTypes = getIssueTypes();
            var currentIssueType = _.find(issueTypes, function(issueType) {
                return issueType.id === issueTypeId;
            });
            if (currentIssueType) {
                var issueIconUrl = currentIssueType.iconUrl;
                $createIssueForm.find(".issue-container img").attr("src", issueIconUrl);
            }
        }
    };
    
    /*** Create Jira Issue ***/

    var removeDisplayMessages = function() {
        $("#create-issue-messages", $createIssueForm).empty();
    };

    var disableForm = function() {
        $("#form-spinner", $createIssueForm).addClass("aui-icon aui-icon-wait");
        $(".create-issue-cancel", $createIssueForm).addClass("hidden");
        $("input,select,textarea,submit", $createIssueForm).disable();
    };

    var getSelectedProjectId = function() {
        return createIssueFormControls.$projectSelect.val();
    };

    var convertFormToJSON = function(issues) {
        var issueFormObject = {
            fields: {
                project: {
                    id: getSelectedProjectId()
                },
                issuetype: {
                    id: createIssueFormControls.$issueTypeSelect.val()
                }
            }
        };

        if ($currentRequiredFields) {
            $currentRequiredFields.each(function(index, requiredField) {
                var $requiredField = $(requiredField);
                issueFormObject.fields[$requiredField.attr("name")] = jiraIntegration.fields.getJSON($requiredField);
            });
        }

        var createIssuesObj = {
            issues: []
        };

        var numOfTruncatedIssues = 0;
        var numOfEditedDescriptions = 0;

        $.each(issues, function(index, issue) {
            if (Confluence.CreateJiraContent.FormTextHelper.isCutLongText(issue.summary)) {
                numOfTruncatedIssues++;
            } else if (!issue.description) {
                // count number of edited descriptions, but don't count case where truncated summary is place in description
                numOfEditedDescriptions++;
            }

            var issueObject = _.clone(issueFormObject);
            issueObject.fields = _.extend({}, issueObject.fields, issue);

            if (currentDescriptionFieldStatus === Confluence.CreateJiraContent.JiraIssue.FieldStatus.REQUIRED &&
                issue.description.length == 0) {
                issueObject.fields.description = issue.summary;
            }
            // try remove issue description when description field is hidden
            if (currentDescriptionFieldStatus === Confluence.CreateJiraContent.JiraIssue.FieldStatus.HIDDEN) {
                delete issueObject.fields.description;
            }

            createIssuesObj.issues.push(issueObject);
        });
        if (numOfTruncatedIssues > 0) {
            Confluence.CreateJiraContent.Analytics.sendAnalyticsForTruncatedSummary(numOfTruncatedIssues);
        }
        if (numOfEditedDescriptions > 0) {
            Confluence.CreateJiraContent.Analytics.sendAnalyticsForDescriptionEdited();
        }
        return JSON.stringify(createIssuesObj);
    };

    var buildErrorMessagesWhenCreateFromTable = function(errorIssues, issueRowIndexs) {
        // get error in each error issue when create
        var errorMessages = [];
        if (errorIssues) {
            $.each(errorIssues, function(index, errorIssue) {
                var errorMessage = AJS.I18n.getText("createjiracontent.dialog.create.issues.result.error",
                                    (issueRowIndexs[errorIssue.failedElementNumber] + 1),
                                    getErrorMessages(errorIssue));
                errorMessages.push(errorMessage);
            });
        }
        return errorMessages;
    };

    var clearCacheInCurrentServer = function() {
        Confluence.CreateJiraContent.DataHelper.clearProjects(createIssueFormControls.$serverSelect.val()); // clear project cached base on serverId
    };

    // OAuthentication handling
    var handleUnauthorizedServer = function() {
        createIssueFormControls.$hiddenItems.addClass("hidden");
        var currentServer = getCurrentServer();
        if (currentServer.authUrl) {
            var onAuthrizedSuccessCallback = function () {
                clearCacheInCurrentServer();
                createIssueFormControls.$hiddenItems.removeClass("hidden");
                $("#jiraserver-issue-messages").empty();
                createIssueFormControls.$jiraEpicContent.empty();
                loadProjects();
            };
            createOauthForm(onAuthrizedSuccessCallback);
        } else {
            // JIRA authenticated configuration was changed when server list are loaded    
            showJiraServerWarning(AJS.I18n.getText('createjiracontent.dialog.form.jiraserver.authurl.error'));
        }
    };
    
    var jiraServerStatus =  function(statusCode, data) {
        //application link is exsited, but do not have any configuration futher
        if (statusCode == 200 && data && data.length === 0) {
            clearCacheInCurrentServer();
            createIssueFormControls.$hiddenItems.addClass("hidden");
            showJiraServerWarning(AJS.format(AJS.I18n.getText('createjiracontent.dialog.form.jiraserver.project.empty'), AJS.escapeHtml(getCurrentServer().name)));
        } else {
            switch (statusCode) {
                case 401: // Unauthorized
                    handleUnauthorizedServer();
                    break;
                case 404: // not found
                case 500: // applink error
                case 504: // proxy timeout
                    createIssueFormControls.$hiddenItems.addClass("hidden");
                    showJiraServerWarning(AJS.I18n.getText('createjiracontent.dialog.form.jiraserver.connect.error'));
                    break;
                default: //normal
                    createIssueFormControls.$hiddenItems.removeClass("hidden");
            }
        }
        refreshDialog();
    };

    var createGetFieldsParamObj = function () {
        return {
            serverId : createIssueFormControls.$serverSelect.val(),
            projectId : createIssueFormControls.$projectSelect.val(),
            issueTypeId : createIssueFormControls.$issueTypeSelect.val(),
            defaultRequiredFields : DEFAULT_REQUIRED_FIELDS
        };
    };

    var createIssues = function(issues, formObject, tableColumnIndex, issueRowIndexs) {
        var $epicCheckbox = $("#epic-link > .checkbox");
        if ($epicCheckbox.is(":checked") && $epicCheckbox.is(":visible")) {
            Confluence.CreateJiraContent.Analytics.sendAnalyticsForCreatedIssueWithEpic();
            Confluence.CreateJiraContent.JiraIssue.getCreateIssueMeta(createGetFieldsParamObj(), function(createMeta) {
                //insert epic link in each issue
                if (createMeta.epicField) {
                    var epicValue = $epicCheckbox.val();
                    $.each(issues, function(index, issue) {
                        issue[createMeta.epicField] = epicValue;
                    });
                }
                executeCreateJiraIssue(issues, formObject, tableColumnIndex, issueRowIndexs);
            });
        } else {
            executeCreateJiraIssue(issues, formObject, tableColumnIndex, issueRowIndexs);
        }
    };

    var getErrorMessages = function(errorIssue) {
        var errorFieldValues = _.values(errorIssue.elementErrors.errors);
        var errorFieldKeys = _.keys(errorIssue.elementErrors.errors);

        var getFieldParamObj = createGetFieldsParamObj();
        for (var i = 0; i < errorFieldKeys.length; i++) {
            var fieldName = Confluence.CreateJiraContent.JiraIssue.getFieldName(errorFieldKeys[i], getFieldParamObj);
            if (fieldName && errorFieldValues[i].indexOf(fieldName) == -1) {
                errorFieldValues[i] = fieldName + ": " + errorFieldValues[i];
            }
        }

        // get and display errors from both errorMessages and errors from response
        var errors = _.union(_.values(errorIssue.elementErrors.errorMessages), errorFieldValues);
        errors = errors.join(', ');
        // some message end with "." so need replace ".," by "," only
        errors = errors.replace(".,", ",");
        return errors;
    };

    var executeCreateJiraIssue = function(issues, formObject, tableColumnIndex, issueRowIndexs) {
        removeDisplayMessages();
        disableForm();

        var jiraRestUrl = Confluence.getContextPath() + "/rest/jira-integration/1.0/issues";

        $.ajax({
            type: "POST",
            contentType: "application/json",
            url: jiraRestUrl + "?applicationId=" + createIssueFormControls.$serverSelect.val(),
            data: convertFormToJSON(issues),
            timeout: 3 * 60 * 1000, // 3 minutes
            success: function(data) {
                if (tableColumnIndex != undefined) {
                    updatePageAfterCreatedIssues(data, formObject, tableColumnIndex, issueRowIndexs);
                } else {
                    if (!data.errors) {
                        insertIssueInPage(data.issues[0], formObject);
                    } else {
                        Confluence.CreateJiraContent.Analytics.sendAnalyticsForCreateError('rest');
                        refreshPageAfterInsertIssues([], false, [getErrorMessages(data.errors[0])]);
                    }
                }
            },
            error: function(xhr) {
                var errorMessages = [];
                var responseErrorIssues = [];
                if (xhr.status == 500) {
                    try {
                        responseErrorIssues = JSON.parse(xhr.responseText);
                        if (responseErrorIssues instanceof Array) {
                            //in case we have multiple error issue, just display error in first issue.
                            $.each(responseErrorIssues[0].errors, function(field, errorMsg) {
                                errorMessages.push(errorMsg);
                            });
                        } else {
                            errorMessages.push(responseErrorIssues);
                        }
                    }
                    catch (e) {
                        responseErrorIssues = [];
                        errorMessages.push("Parse error:" + e.message);
                    }
                } else {
                    errorMessages.push(AJS.I18n.getText('createjiracontent.dialog.create.issue.error.server'));
                }
                Confluence.CreateJiraContent.Analytics.sendAnalyticsForCreateError('xhr');
                refreshPageAfterInsertIssues(responseErrorIssues, false, errorMessages);
            }
        });
    };

    /**
     * insert created issue single mode
     * @param issue
     * @param: formObject
     */
    var insertIssueInPage = function(issue, formObject) {
        if (formObject.isInSortedTable() && formObject.getSelectionObject().searchText.numMatches != 1) {
            refreshPageAfterInsertIssues([ issue ], false, []);
            return;
        }
        var insertionBean = Confluence.HighlightAction.createInsertionBean(
            [{xmlInsertion: toIssueMacroXml(issue)}], formObject.getSelectionObject().searchText);

        Confluence.CreateJiraContent.Analytics.sendAnalyticsForCreatingIssue('text');

        Confluence.HighlightAction.insertContentAtSelectionEnd(insertionBean)
            .done(function(data) {
                if (!data) {
                    Confluence.CreateJiraContent.Analytics.sendAnalyticsForJIMInsertFailed('text', 'algorithm');
                }
                refreshPageAfterInsertIssues([issue], data, []);
            })
            .fail(function(response) {
                Confluence.CreateJiraContent.Analytics.sendAnalyticsForJIMInsertFailed('text', 'server');
                refreshPageAfterInsertIssues([issue], false, [], response.statusText);
            });
    };

    var toIssueMacroXml = function(issue) {
        var currentServer = getCurrentServer();
        var jiraMacro = Confluence.CreateJiraContent.issueMacroXml({
            "showSummary" : false,
            "serverName" : currentServer.name,
            "serverId" : currentServer.id,
            "issueKey" : issue.issue.key
        });
        return jiraMacro;
    };

    /**
     * insert created issue from creating issue in table mode
     * @param paramObj: {createdIssueRowIndexes, createdIssues, errorMessages, formObject, tableColumnIndex}
     */
    var insertCreatedIssuesInPage = function(paramObj) {
        if (paramObj.formObject.isInSortedTable()) {
            refreshPageAfterInsertIssues(paramObj.createdIssues, false, paramObj.errorMessages);
            return;
        }
        
        var cellXmlInsertions = [];
        // Prepare jira macro xml content for insert into table and created
        // issues messages
        $.each(paramObj.createdIssues, function(index, createdIssue) {
            var cellXmlInsertion = {};
            cellXmlInsertion.rowIndex = paramObj.createdIssueRowIndexes[index];
            cellXmlInsertion.xmlInsertion = toIssueMacroXml(createdIssue);
            cellXmlInsertions.push(cellXmlInsertion);
        });

        var tableInsertionBean = Confluence.HighlightAction.createTableInsertionBean(cellXmlInsertions, paramObj.tableColumnIndex,
            paramObj.formObject.getSelectionObject().searchText);

        Confluence.CreateJiraContent.Analytics.sendAnalyticsForCreatingIssue('table', paramObj.createdIssues.length);

        Confluence.HighlightAction.insertContentsInTableColumnCells(tableInsertionBean)
            .done(function(data) {
                if (!data) {
                    Confluence.CreateJiraContent.Analytics.sendAnalyticsForJIMInsertFailed('table', 'algorithm');
                }
                refreshPageAfterInsertIssues(paramObj.createdIssues, data, paramObj.errorMessages);
            }).fail(function(response) {
                Confluence.CreateJiraContent.Analytics.sendAnalyticsForJIMInsertFailed('table', 'server');
                refreshPageAfterInsertIssues(paramObj.createdIssues, false, paramObj.errorMessages, response.statusText);
            });
    };

    var refreshPageAfterInsertIssues = function(createdIssues, addedToPage, errorMessages, statusText) {
        var issuesURL = getCurrentServer().url;
        if (createdIssues.length == 1) {
            issuesURL += "/browse/" + createdIssues[0].issue.key;
        } else {
            var jql = "key in (";
            jql += $.map(createdIssues, function(issue) {
                return issue.issue.key;
            }).join(",");
            jql += ")";
            issuesURL += "/issues/?jql=" + encodeURIComponent(jql);
        }
        var currentPath = window.location.href.split("#")[0];
        var oldJIRAIssueCreatedIdx = currentPath.indexOf("JIRAIssuesCreated");
        if (oldJIRAIssueCreatedIdx > 0) {
            currentPath = currentPath.substring(0, oldJIRAIssueCreatedIdx - 1);
        }
        var reloadURL = currentPath
            + (currentPath.indexOf("?") > 0 ? "&" : "?")
            + "JIRAIssuesCreated=true"
            + "&numOfIssues=" + createdIssues.length
            + "&issuesURL=" + encodeURIComponent(issuesURL)
            + "&addedToPage=" + addedToPage;
        if (createdIssues.length > 0) {
            reloadURL += "&issueId=" + encodeURIComponent(createdIssues[0].issue.key);
        }
        if(statusText != undefined) {
            reloadURL += "&statusText=" + statusText;
        }
        $.each(errorMessages, function(index, message) {
            reloadURL += "&errorMessages=" + message;
        });
        $("#form-spinner").removeClass("aui-icon aui-icon-wait");
        Confluence.CreateJiraContent.CreateIssue.getDialogObject().hide();
        window.location.replace(reloadURL);
    };

    var updatePageAfterCreatedIssues = function(data, formObject, tableColumnIndex, issueRowIndexes) {
        var createdIssueRowIndexes = [];
        $.each(data.issues, function(index, issue) {
            createdIssueRowIndexes.push(issueRowIndexes[issue.elementNumber]);
        });

        var errorMessages = buildErrorMessagesWhenCreateFromTable(data.errors, issueRowIndexes);
        if (data.issues.length) {
            insertCreatedIssuesInPage({
                createdIssues: data.issues,
                createdIssueRowIndexes: createdIssueRowIndexes,
                tableColumnIndex: tableColumnIndex,
                formObject: formObject,
                errorMessages: errorMessages});
        } else if (data.errors.length) {
            refreshPageAfterInsertIssues([], false, errorMessages);
        }
    };

    var tooltipFormat = function(result) {
        return Confluence.CreateJiraContent.selectOption({"optionValue": result.text});
    };

    var issueTypeSelectFormat = function(result) {
        var issueTypeImageUrl;
        if (result) {
            if (result.id >= 0) {
                issueTypeImageUrl = Confluence.CreateJiraContent.DataHelper.getIssueTypeIconUrl(
                                        createIssueFormControls.$serverSelect.val(),
                                        createIssueFormControls.$projectSelect.val(), result.id);
            }
            return Confluence.CreateJiraContent.selectOptionWithImage({
                "optionValue": result.text,
                "imageUrl": issueTypeImageUrl
            });
        }
    };

    var projectSelectFormat = function(result) {
        var projectAvatarUrl;
        if (result) {
            if (result.id >= 0) {
                projectAvatarUrl = Confluence.CreateJiraContent.DataHelper.getProjectIconUrl(createIssueFormControls.$serverSelect.val(), result.id);
            }
            return Confluence.CreateJiraContent.selectOptionProject({
                "optionValue": result.text,
                "imageUrl": projectAvatarUrl
            });
        }
    };

    var getCurrentServer = function() {
        return Confluence.CreateJiraContent.DataHelper.getServerById(createIssueFormControls.$serverSelect.val());
    };

    var getCurrentProject = function() {
        var serverId = createIssueFormControls.$serverSelect.val();
        var projectId = createIssueFormControls.$projectSelect.val();
        var projects = Confluence.CreateJiraContent.DataHelper.getProjects(serverId);
        return _.find(projects, function(project) {
            return project.id === projectId;
        });
    };

    var getCurrentJiraCreateIssueUrl = function() {
        var jiraCreateIssueUrl = getCurrentServer().url + "/secure/CreateIssueDetails!Init.jspa?pid="
                + getCurrentProject().id + "&issuetype=" + createIssueFormControls.$issueTypeSelect.val();
        var summaryValue =  createIssueFormControls.$summaryField.val();
        if (summaryValue.length) {
            jiraCreateIssueUrl = jiraCreateIssueUrl + "&summary=" + encodeURIComponent(summaryValue);
        }
        if (currentDescriptionFieldStatus !== Confluence.CreateJiraContent.JiraIssue.FieldStatus.HIDDEN) {
            var descriptionValue = createIssueFormControls.$descriptionField.val();
            if (descriptionValue.length) {
                jiraCreateIssueUrl = jiraCreateIssueUrl + "&description=" + encodeURIComponent(descriptionValue);
            }
        }
        return jiraCreateIssueUrl;
    };

    var showJiraServerWarning = function(message) {
        AJS.messages.warning($("#jiraserver-issue-messages"), {
            body : message,
            closeable : false
        });

        if (!$createIssueForm.find('#text-suggestion').is(':visible')) {
            $('#inline-dialog-create-issue-dialog > #arrow-create-issue-dialog').removeClass(BOTTOM_ARROW_CLASS);
        }
    };
    
    var createOauthForm = function(successCallback){
        var currentServer = getCurrentServer();
        var oauthCallbacks = {
            onSuccess : function() {
                currentServer.authUrl = null;
                successCallback();
            },
            onFailure : function() {
            }
        };
        showJiraServerWarning(Confluence.CreateJiraContent.getOAuthMessage({'confluenceApplicationName' : 'Confluence'}));
        $('.oauth-init', $('#jiraserver-issue-messages')).click(function(e) {
            AppLinks.authenticateRemoteCredentials(currentServer.authUrl, oauthCallbacks.onSuccess, oauthCallbacks.onFailure);
            e.preventDefault();
        });
    };

    var bindHideEventToDialog = function() {
        createIssueFormControls.$serverSelect.auiSelect2("close");
        createIssueFormControls.$projectSelect.auiSelect2("close");
        createIssueFormControls.$issueTypeSelect.auiSelect2("close");

        // Save current Value for next showing if the dialog is hidden by nature, not by clicking on Cancel button.
        if(Confluence.CreateJiraContent.CreateIssue.getDialogObject().isCancelButtonClicked !== true) {
            saveFormValues(createGetFieldsParamObj());
        }
    };
    /**
     * Storing input value of creating issue and display it for next time open dialog
     **/
    var saveFormValues = function(values) {
        if (values.serverId && values.projectId && values.issueTypeId) {
            storageManager.setItem(FORM_VALUE_STORAGE_KEY, JSON.stringify(values));
        }
    };

    /**
     * Return form's value as Object
     * FALSE if values is null or invalid
     * StoredValue Object if it's valid
     *
     * @param String key
     */
    var loadFormValues = function(key) {
        // Json string may not valid, so it would be good to wrap it inside a try/catch
        try {
            key = key || FORM_VALUE_STORAGE_KEY;
            var selectedValue = JSON.parse(storageManager.getItem(key));

            if (selectedValue && selectedValue.serverId && selectedValue.projectId && selectedValue.issueTypeId) {
                return selectedValue;
            } else {
                return false;
            }
        } catch(err) {
            return false;
        }
    };

    /**
     * Set True to show SelectedValue and Hide two dropdowns
     * @param boolean visible
     */
    var setSummaryVisible = function(visible) {
        var $projectAndType = $createIssueForm.find(".dialog-content > .main-field"),
            $summaryEl = $createIssueForm.find(".dialog-content .summary-stored-value");

        if (visible) {
            var selectedProjectId = createIssueFormControls.$projectSelect.val();
            var selectedIssueTypeId = createIssueFormControls.$issueTypeSelect.val();
            // We only call summaryContent and show Summary when Project Select has already selected
            if (selectedProjectId != -1 && selectedProjectId != null && selectedIssueTypeId != -1 && selectedIssueTypeId != -1) {
                var summaryContent = extractSummaryContent();
                if (summaryContent) {
                    // bind text into summary
                    $summaryEl.remove();

                    var summaryHTML = Confluence.CreateJiraContent.summaryStoredValue({
                        storedValue: summaryContent
                    });
                    $(summaryHTML).insertBefore($createIssueForm.find('#issue-content'));
                    $projectAndType.addClass("hidden");
                }
            }
        } else {
            $projectAndType.removeClass("hidden");
            $summaryEl.addClass("hidden");
        }

        // We have to call refresh to make sure the dialog still look good
        refreshDialog();
    };

    /**
     * Extract information for Summary Div
     * @returns Object
     * Object = {
     *     projectText: <STRING>,
     *     issueTypeIcon: <STRING>,
     *     projectIcon: jQuery Object,
     *     issueTypeIcon: jQuery Object
     * }
     */
    var extractSummaryContent = function(){
        var $projectSelect = createIssueFormControls.$projectSelect,
            $issueTypeSelect = createIssueFormControls.$issueTypeSelect;

        // get project name and issueType name from select
        var projectText = $projectSelect.auiSelect2('data').text,
            issueTypeText = $issueTypeSelect.auiSelect2('data').text;

        // get Icon
        var $projectIcon = $projectSelect.auiSelect2("container").find('.select2-chosen img'),
            $issueTypeIcon = $issueTypeSelect.auiSelect2("container").find('.select2-chosen img');

        if (projectText && issueTypeText && $projectIcon && $issueTypeIcon) {
            return {
                projectText: projectText,
                issueTypeText: issueTypeText,
                projectIcon: $projectIcon.attr("src"),
                issueTypeIcon: $issueTypeIcon.attr("src")
            };
        } else {
            return null;
        }
    };

    /**
     * Check Select control if it has an option with special value or not
     * @param jQueryObject $select
     * @param String/Integer value
     * @returns True/False
     */
    var isOptionAvailable = function($select, value) {
        return ($select.find('option[value="'+ value +'"]').length);
    };

    /**
     * Set selected value into a Select2 control
     * @param jQueryObject $select
     * @param String/Integer value
     * @returns {boolean}
     */
    var setSelectFieldValue = function($select, value) {
        if (isOptionAvailable($select, value)) {
            $select.auiSelect2("val", value);
            return true;
        } else {
            storedDialogValues = false;
            return false;
        }
    };

    /**
     * Try to resolve epicIssueType and detect epicLink on page 
     */
    var detectEpicIssueOnPage = function(serverId) {
        var onSuccess = function(issueTypeId) {
            epicIssueTypeId = issueTypeId;
            epicLinkJiraIssuePageHandling(serverId);
        };
        var onError = function() {
            epicIssueTypeId = (serverId === getCurrentServer().id) ? null : epicIssueTypeId;
        };
        Confluence.CreateJiraContent.JiraIssue.resolveEpicIssueType(serverId, onSuccess, onError);
    };
    
    /**
     * Detect if there is any Epic issue on the page with match with server selection
     * If have 1 ->display on Create-Jira-Issue dialog
     * If 0 or more than 2: ignore it
     */
    var epicLinkJiraIssuePageHandling = function(serverId) {
        // If xhrEpicPage is existed, this mean previous ajax hadn't completed yet, call abort() to cancel it.
        if (xhrEpicPage && xhrEpicPage.readyState !== 4 && xhrEpicPage.abort) {
            xhrEpicPage.abort();
        }
        var $jiraEpicContent = createIssueFormControls.$jiraEpicContent;

        xhrEpicPage = $.ajax({
            type : "GET",
            timeout: 0,
            contentType : "application/json",
            url : AJS.Confluence.getContextPath() + '/rest/createjiracontent/1.0/find-epic-issue',
            data : {
                pageId : AJS.params.pageId,
                serverId : serverId,
                epicIssueTypeId: epicIssueTypeId
            },
            success : function(epicData) {
                if (!epicData || !epicData.epicKey) {
                    $jiraEpicContent.empty();
                    return;
                }
                var epicIssueHtml = Confluence.CreateJiraContent.renderEpicContent({
                    epicKey: epicData.epicKey,
                    epicHtmlPlaceHolder : epicData.epicHtmlPlaceHolder
                });
                $jiraEpicContent.html(epicIssueHtml);
                // get summary data before it removed out dom.
                epicSummaryText = $jiraEpicContent.find(".summary").text();
                //remove summary, status out of epic render
                var epicEl = $jiraEpicContent.find('a:first-child').clone();
                $jiraEpicContent.find('.jira-issue').html(epicEl);
                addEpicLinkTooltip($jiraEpicContent);
                refreshDialog();
            },
            error : function(xhr) {
                $jiraEpicContent.empty();
                AJS.logError('confluence-jira-content:epicLinkJiraIssuePageHandling - Error when detect epic on page with status=' + xhr.status);
            }
        });
    };

    var addEpicLinkTooltip = function($jiraEpicContent) {
        $jiraEpicContent.find(".jira-issue").attr("title", epicSummaryText).tooltip({gravity: 'sw'});
    };

    var removeDialogLoading = function() {
        $createIssueForm.find("#create-issue-loading").remove();
    };

    var handleServerLoadingSuccess = function() {
        removeDialogLoading();
        var serverId = createIssueFormControls.$serverSelect.val();
        // do not load project when server is empty
        if(serverId) {
            // display create isue dialog content when finished loading server data
            $createIssueForm.find("#jira-servers, .dialog-content, #prepare-issue-messages, .buttons-group, #text-suggestion").removeClass("hidden");
            createIssueFormControls.$jiraEpicContent.empty();
            loadProjects();
        }
    };

    var handleServerLoadingError = function() {
        removeDialogLoading();
        showJiraServerWarning(AJS.I18n.getText("createjiracontent.dialog.form.get.jiraserver.error"));
    };

    var refreshDialog = function() {
        if (Confluence.CreateJiraContent.CreateIssue.getDialogObject()) {
            Confluence.CreateJiraContent.CreateIssue.getDialogObject().refresh();
        }
    };

    return {
        init : function($form) {
            init($form);
            createSelect2WithTooltip(createIssueFormControls.$projectSelect,
                {
                    "containerCssClass": "select-container select-project-container",
                    "dropdownCssClass": "projects-dropdown",
                    "formatSelection": projectSelectFormat,
                    "formatResult": projectSelectFormat
                }
            );
            createSelect2WithTooltip(createIssueFormControls.$issueTypeSelect,
                {
                    "containerCssClass": "select-container select-issuetype-container",
                    "dropdownCssClass": "issue-types-dropdown",
                    "minimumResultsForSearch": -1,
                    "formatSelection": issueTypeSelectFormat,
                    "formatResult": issueTypeSelectFormat
                }
            );

            enableSubmitButton(false);
            clearAndDisableSelectBox(createIssueFormControls.$issueTypeSelect);
            storedDialogValues = loadFormValues(FORM_VALUE_STORAGE_KEY);
            bindElementEvents();
            setSummaryVisible(storedDialogValues !== false);
            loadServers().pipe(handleServerLoadingSuccess, handleServerLoadingError);
        },
        createIssues: createIssues,
        validateCreateIssueForm: validateCreateIssueForm,
        removeDisplayMessages: removeDisplayMessages,
        bindHideEventToDialog: bindHideEventToDialog,
        changeIssuesTypeIconForPreviewPanel: changeIssuesTypeIconForPreviewPanel
    };
})(AJS.$, window._);
